<?php

namespace Drupal\dblog_persistent;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\dblog_persistent\Entity\ChannelInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of Persistent Log Message Type entities.
 */
class ChannelListBuilder extends ConfigEntityListBuilder {

  /**
   * @var \Drupal\dblog_persistent\DbLogPersistentStorageInterface
   */
  protected $logStorage;

  /**
   * DbLogPersistentTypeListBuilder constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   * @param \Drupal\dblog_persistent\DbLogPersistentStorageInterface $loader
   */
  public function __construct(EntityTypeInterface $entity_type,
                              EntityStorageInterface $storage,
                              DbLogPersistentStorageInterface $loader) {
    parent::__construct($entity_type, $storage);
    $this->logStorage = $loader;
  }

  public static function createInstance(ContainerInterface $container,
                                        EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('dblog_persistent.storage')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['label'] = $this->t('Name');
    $header['types'] = $this->t('Filters');
    $header['count'] = $this->t('Messages');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\dblog_persistent\Entity\ChannelInterface $entity */
    $row['label'] = $entity->label();
    $filters = [];
    $filters[] = [
      '#theme' => 'item_list',
      '#prefix' => $this->t('Types: '),
      '#context' => ['list_style' => 'comma-list'],
      '#items' => $entity->getTypes(),
      '#empty' => '*',
    ];
    $filters[] = [
      '#theme' => 'item_list',
      '#prefix' => $this->t('Severity: '),
      '#context' => ['list_style' => 'comma-list'],
      '#items' => \array_intersect_key(RfcLogLevel::getLevels(), $entity->getLevels()),
      '#empty' => '*',
    ];
    if ($message = $entity->getMessage()) {
      $filters[] = $this->t('Message contains: %text', [
        '%text' => $message,
      ]);
    }
    $row['summary']['data'] = [
      '#theme' => 'item_list',
      '#items' => $filters,
    ];

    $row['count'] = $this->logStorage->countChannel($entity->id());
    if ($row['count']) {
      $row['count'] = ['data' => [
        '#type' => 'link',
        '#url' => $entity->toUrl('canonical'),
        '#title' => $row['count'],
      ]];
    }

    return $row + parent::buildRow($entity);
  }

  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);

    // If the channel is not empty, we can clear it.
    if ($this->logStorage->countChannel($entity->id())) {
      $operations['clear'] = [
        'title' => $this->t('Clear'),
        'url' => $entity->toUrl('clear-form'),
        'weight' => 20,
      ];
    }
    return $operations;
  }

}
