<?php

namespace Drupal\dblog_persistent\Form;

use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\dblog_persistent\DbLogPersistentStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class DBLogPersistentTypeForm.
 */
class ChannelForm extends EntityForm {

  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * Message types.
   *
   * @var string
   */
  protected $messageTypes = [];

  /**
   * @var \Drupal\dblog_persistent\DbLogPersistentStorageInterface
   */
  protected $loader;

  /**
   * DBLogPersistentTypeForm constructor.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityStorageInterface $storage
   * @param \Drupal\dblog_persistent\DbLogPersistentStorageInterface $loader
   */
  public function __construct(ConfigEntityStorageInterface $storage,
                              DbLogPersistentStorageInterface $loader) {
    $this->storage = $storage;
    $this->loader = $loader;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('dblog_persistent_channel'),
      $container->get('dblog_persistent.storage')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\dblog_persistent\Entity\ChannelInterface $entity */
    $entity = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => [$this, 'exists'],
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['info']['#markup'] = $this->t('Configure the filters that are applied to this log channel.');

    $form['types'] = [
      '#type' => 'select',
      '#title' => $this->t('Types'),
      '#multiple' => TRUE,
      '#options' => $this->getMessageTypes() + array_combine($entity->getTypes(), $entity->getTypes()),
      '#default_value' => $entity->getTypes(),
      '#description' => $this->t('Record messages of specific types.'),
    ];

    $form['levels'] = [
      '#type' => 'select',
      '#title' => $this->t('Severity'),
      '#multiple' => TRUE,
      '#options' => RfcLogLevel::getLevels(),
      '#default_value' => $entity->getLevels(),
      '#description' => $this->t('Record messages with specific severities.'),
    ];

    $form['message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Message'),
      '#default_value' => $entity->getMessage(),
      '#description' => $this->t('Record messages that contain a substring.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\Exception\UndefinedLinkTemplateException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $status = $entity->save();

    if ($status === SAVED_NEW) {
      drupal_set_message($this->t('Created the %label Persistent Log Message Type.',
        [
          '%label' => $entity->label(),
        ]));
    }
    else {
      drupal_set_message($this->t('Saved the %label Persistent Log Message Type.',
        [
          '%label' => $entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($entity->toUrl('collection'));
  }

  /**
   * Check if a given ID exists.
   *
   * @param string $id
   *
   * @return bool
   */
  public function exists(string $id): bool {
    return $this->storage->load($id) !== NULL;
  }

  /**
   * List all available message types.
   */
  private function getMessageTypes() {
    if (!$this->messageTypes) {
      foreach (_dblog_get_message_types() as $type) {
        $this->messageTypes[$type] = $type;
      }
      foreach ($this->loader->getTypes() as $type) {
        $this->messageTypes[$type] = $type;
      }
    }

    return $this->messageTypes;
  }


}
